from flask import Flask, render_template, request, redirect, url_for
import psycopg2
import os

app = Flask(__name__)

# Configure Flask-related environment variables
app.config['FLASK_APP'] = os.environ.get('FLASK_APP', 'app.py')
app.config['FLASK_RUN_HOST'] = os.environ.get('FLASK_RUN_HOST', '127.0.0.1')
app.config['FLASK_RUN_PORT'] = int(os.environ.get('FLASK_RUN_PORT', 5000))
app.config['FLASK_ENV'] = os.environ.get('FLASK_ENV', 'development')
app.config['DEBUG'] = os.environ.get('FLASK_DEBUG', app.config['FLASK_ENV'] == 'development')


# Configure your PostgreSQL connection
db_config = {
    'host': os.environ.get('DB_HOST', 'localhost'),
    'database': os.environ.get('DB_NAME', 'postgres'),
    'user': os.environ.get('DB_USER', 'postgres'),
    'password': os.environ.get('DB_PASSWORD', 'mysecretpassword'),
}


# Function to get a list of songs from the database
def get_songs(search_term=None):
    connection = psycopg2.connect(**db_config)
    cursor = connection.cursor()

    query = """
        SELECT DISTINCT s.id, s.title, s.artist, s.play_count, q.status
        FROM ultra_star_song s
        LEFT JOIN queue q ON s.id = q.song_id
    """

    if search_term:
        query += " WHERE s.title ILIKE %s OR s.artist ILIKE %s"
        cursor.execute(query + " ORDER BY s.title", (f"%{search_term}%", f"%{search_term}%"))
    else:
        cursor.execute(query + " ORDER BY s.title")

    songs = cursor.fetchall()
    connection.close()
    return songs


# Function to get the queue of songs from the database
def get_queue():
    connection = psycopg2.connect(**db_config)
    cursor = connection.cursor()
    cursor.execute(
        "SELECT q.id, s.title, s.artist, s.play_count, q.status FROM queue q JOIN ultra_star_song s ON q.song_id = s.id ORDER BY q.position")
    queue = cursor.fetchall()
    connection.close()
    return queue


# Route to display the list of songs and the queue
@app.route('/')
def index():
    search_term = request.args.get('search', '')
    songs = get_songs(search_term)
    queue = get_queue()
    return render_template('index.html', songs=songs, search_term=search_term, queue=queue)


# Route to handle song addition to the queue
@app.route('/add_to_queue/<int:song_id>', methods=['POST'])
def add_to_queue(song_id):
    add_song_to_queue(song_id)
    return redirect(url_for('index'))


# Route to handle song removal from the queue
@app.route('/remove_from_queue/<int:queue_id>', methods=['POST'])
def remove_from_queue(queue_id):
    remove_song_from_queue(queue_id)
    return redirect(url_for('queue'))


# Route to display the queue
@app.route('/queue')
def queue():
    queue = get_queue()
    return render_template('queue.html', queue=queue)


# Function to add a song to the queue
def add_song_to_queue(song_id):
    connection = psycopg2.connect(**db_config)
    cursor = connection.cursor()

    # Increment the play count in the songs table
    cursor.execute("UPDATE ultra_star_song SET play_count = play_count + 1 WHERE id = %s", (song_id,))

    # Add the song to the queue
    cursor.execute(
        "INSERT INTO queue (song_id, position) VALUES (%s, (SELECT COALESCE(MAX(position), 0) + 1 FROM queue))",
        (song_id,))

    connection.commit()
    connection.close()


# Function to remove a song from the queue
def remove_song_from_queue(queue_id):
    connection = psycopg2.connect(**db_config)
    cursor = connection.cursor()
    cursor.execute("DELETE FROM queue WHERE id = %s", (queue_id,))
    connection.commit()
    connection.close()


# Function to update the order of songs in the queue
def update_queue_order(new_order):
    connection = psycopg2.connect(**db_config)
    cursor = connection.cursor()

    for position, queue_id in enumerate(new_order, 1):
        cursor.execute("UPDATE queue SET position = %s WHERE id = %s", (position, queue_id))

    connection.commit()
    connection.close()


if __name__ == '__main__':
    app.run(host=app.config['FLASK_RUN_HOST'], port=app.config['FLASK_RUN_PORT'], debug=app.config['DEBUG'])

